#!/usr/bin/env python

# Import sphere functionality
import sphere
import numpy
import matplotlib.pyplot as plt
from sklearn.linear_model import LinearRegression

# Number of particles
np = 1e4

# Common simulation id
sim_id = "hz"

# Deviatoric stress [Pa]
Nlist = numpy.array([51e3, 101e3, 202e3, 303e3, 404e3])

### INITIALIZATION ###

# New class
sim = sphere.sim(np = np, nd = 3, nw = 0, sid = sim_id + "-init")

N_avg = 50
tau_c = []

for N in Nlist:
    sim.id("{}-shear-N{}".format(sim_id, N))
    lastf = sim.status()
    #sim.readlast()
    #sim.sheardisp()

    # average shear stress over last files
    tau_c_curr = 0.0

    for i in range(N_avg):
        sim.readstep(lastf - i)
        tau_c_curr += sim.shearStress('effective')
    tau_c.append(tau_c_curr/N_avg)

fig = plt.figure()
tau_c = numpy.array(tau_c)
tau_c /= 1e3
Nlist /= 1e3
plt.plot(Nlist, tau_c, '+')
mc = LinearRegression().fit(Nlist.reshape((-1, 1)), tau_c)
Nspace = numpy.linspace(0.0, Nlist[-1], endpoint=True)
plt.plot(Nspace, mc.predict(Nspace.reshape(-1,1)), '-')
plt.title("friction = {:.2}, cohesion = {:.2} kPa".format(mc.coef_[0], mc.intercept_))
plt.xlabel("Effective normal stress [kPa]")
plt.ylabel("Shear stress [kPa]")
plt.savefig("hz-mohr-coulomb.pdf")
