#!/usr/bin/env python

import sphere
import numpy as np
import matplotlib
matplotlib.rcParams.update({'font.size': 12})
import matplotlib.pyplot as plt
import subprocess

dpdz_values = [0.0, -50.0, -100.0]
slope_angle_values = [5.0, 10.0, 15.0, 20.0]

scatter_color = '#666666'
scatter_alpha = 0.6
dp_dz_vals = np.empty(len(dpdz_values)*len(slope_angle_values))
slope_angles = np.empty_like(dp_dz_vals)
fluxes = np.empty_like(dp_dz_vals)

'''
j = 0
for dpdz in dpdz_values:
    outfiles = ''
    for slope_angle in slope_angle_values:

        sim = sphere.sim("supraglacial-slope{}-dpdz{}".format(slope_angle, dpdz),
                         fluid=True)
        print('### ' + sim.id())
        print('Last output file: ' + str(sim.status()))
        sim.readTime(4.99)

        fig = plt.figure()

        title = 'slope = ' + str(slope_angle) + '$^\circ$, ' + \
                '$dp/dz$ = -' + str(dpdz) + ' Pa/m'

        z = np.zeros(20)
        v_x_space_avg = np.empty_like(z)
        xsum_space_avg = np.empty_like(z)
        dz = np.max(sim.x[:,2])/len(z)
        for i in range(len(z)):
            z[i] = i*dz + 0.5*dz
            I = np.nonzero((sim.x[:,2] >= i*dz) & (sim.x[:,2] < (i+1)*dz))
            v_x_space_avg[i] = np.mean(sim.vel[I,0])
            xsum_space_avg[i] = np.mean(sim.xyzsum[I,0])

        plt.plot(sim.vel[:,0], sim.x[:,2], '.',
                 color=scatter_color, alpha=scatter_alpha)
        plt.plot(v_x_space_avg, z, '+-k')
        plt.title(title)
        plt.xlabel('Horizontal particle velocity $v_x$ [m/s]')
        plt.ylabel('Vertical position $z$ [m]')
        plt.savefig(sim.id() + '-vel.pdf')
        plt.savefig(sim.id() + '-vel.png')
        plt.clf()

        plt.plot(sim.xyzsum[:,0]/sim.time_current, sim.x[:,2], '.',
                 color=scatter_color, alpha=scatter_alpha)
        plt.plot(xsum_space_avg/sim.time_current, z, '+-k')
        plt.title(title)
        plt.xlabel('Average horizontal particle velocity $\\bar{v}_x$ [m/s]')
        plt.ylabel('Vertical position $z$ [m]')
        plt.savefig(sim.id() + '-avg_vel.pdf')
        plt.savefig(sim.id() + '-avg_vel.png')
        plt.clf()

        plt.close()
        outfiles += sim.id() + '-avg_vel.png '

        dp_dz_vals[j] = dpdz
        slope_angles[j] = slope_angle
        fluxes[j] = np.trapz(xsum_space_avg/sim.time_current, dx=dz)
        j += 1

    subprocess.call('montage ' + outfiles +
                    '-geometry +0+0 ' +
                    'supraglacial-avg_vel-dpdz_' + str(dpdz) + '.png ',
                     shell=True)

for dpdz in dpdz_values:
    I = np.nonzero(dp_dz_vals == dpdz)
    plt.semilogy(slope_angles[I[0]], fluxes[I[0]], '+-', label='$dp/dz$ = ' + str(dpdz) + ' Pa/m')

plt.legend()
plt.xlabel('Slope [$^\circ$]')
plt.ylabel('Specific sediment flux [m$^2$/s]')
plt.savefig('supraglacial_flux.png')
plt.savefig('supraglacial_flux.pdf')
'''

# time series
for dpdz in dpdz_values:
    for slope_angle in slope_angle_values:
        sim = sphere.sim("supraglacial-slope{}-dpdz{}".format(slope_angle, dpdz), fluid=True)
        print('### ' + sim.id())
        sim.readlast()

        N_time = min(100, sim.status())
        timesteps = np.linspace(sim.time_file_dt[0], sim.time_current[0] - sim.time_file_dt[0], N_time)
        porosity = np.empty(N_time)
        velocity = np.empty(N_time)
        displacement = np.empty(N_time)
        flux = np.empty(N_time)
        z = np.zeros(20)
        v_x_space_avg = np.empty_like(z)
        xsum_space_avg = np.empty_like(z)

        for it in np.arange(N_time):
            sim.readTime(timesteps[it])

            dz = np.max(sim.x[:,2])/len(z)
            for i in range(len(z)):
                z[i] = i*dz + 0.5*dz
                I = np.nonzero((sim.x[:,2] >= i*dz) & (sim.x[:,2] < (i+1)*dz))
                xsum_space_avg[i] = np.mean(sim.xyzsum[I,0])

            porosity[it] = np.mean(sim.phi)
            velocity[it] = np.mean(sim.vel[:,0])
            displacement[it] = np.mean(sim.xyzsum[:,0])
            flux[it] = np.trapz(xsum_space_avg/sim.time_current, dx=dz)

        fig = plt.figure(figsize=(6,8))
        fig.suptitle('slope = ' + str(slope_angle) + '$^\circ$, ' + \
                     '$dp/dz$ = -' + str(dpdz) + ' Pa/m',
					 horizontalalignment='left')

        ax1 = plt.subplot(3,1,1)
        plt.plot(timesteps, porosity, '-')
        plt.ylabel('Porosity [-]')
        plt.setp(ax1.get_xticklabels(), visible=False)

        #ax2 = plt.subplot(3,1,2)
        #plt.semilogy(timesteps, velocity, '-')
        #plt.ylabel('Avg. velocity [m/s]')
        #plt.setp(ax2.get_xticklabels(), visible=False)

        ax3 = plt.subplot(3,1,2)
        plt.semilogy(timesteps, displacement, '-')
        plt.ylabel('Avg. displacement [m]')
        plt.setp(ax3.get_xticklabels(), visible=False)

        ax1 = plt.subplot(3,1,3)
        plt.semilogy(timesteps, flux, '-')
        plt.ylabel('Cumulative flux [m$^2$/s]')
        plt.xlabel('Time [s]')

        plt.tight_layout()
        plt.savefig(sim.id() + '-timeseries.png')
        plt.savefig(sim.id() + '-timeseries.pdf')
        plt.clf()
        plt.close()
